window.Gleam.initSearch([{"doc":"glats","title":"glats","content":"# glats\n\n[![Package Version](https://img.shields.io/hexpm/v/glats)](https://hex.pm/packages/glats)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/glats/)\n\nA NATS client for Gleam. This wraps Elixir&#39;s client, [Gnat](https://hex.pm/packages/gnat).\n\n## Publish\n\n```gleam\nimport gleam/result\nimport glats\n\npub fn main() {\n  use conn &lt;- result.then(glats.connect(&quot;localhost&quot;, 4222, []))\n\n  // Publish a single message to &quot;some.topic&quot;.\n  let assert Ok(Nil) = glats.publish(conn, &quot;some.topic&quot;, &quot;hello world!&quot;, [])\n\n  Ok(Nil)\n}\n```\n\n## Subscribe\n\n```gleam\nimport gleam/result\nimport gleam/erlang/process\nimport glats\n\npub fn main() {\n  use conn &lt;- result.then(glats.connect(&quot;localhost&quot;, 4222, []))\n\n  let subject = process.new_subject()\n\n  // Subscribe to &quot;some.topic&quot;.\n  // Messages will be delivered to the erlang subject passed in.\n  let assert Ok(sid) = glats.subscribe(conn, subject, &quot;some.topic&quot;, [])\n\n  // Publish a single message to &quot;some.topic&quot;.\n  let assert Ok(Nil) = glats.publish(conn, &quot;some.topic&quot;, &quot;hello world!&quot;, [])\n\n  // Receive from erlang subject.\n  let assert Ok(glats.ReceivedMessage(\n    conn: _conn,\n    sid: _sid,\n    status: _status,\n    message: glats.Message(\n      topic: _topic,\n      headers: _headers,\n      reply_to: _reply_to,\n      body: _body,\n    ),\n  )) = process.receive(subject, 1000)\n\n  // Unsubscribe from the subscription.\n  let assert Ok(Nil) = glats.unsubscribe(conn, sid)\n\n  Ok(Nil)\n}\n```\n\n## Request handler\n\n```gleam\nimport gleam/io\nimport gleam/option.{None}\nimport gleam/result\nimport gleam/erlang/process\nimport glats\nimport glats/handler.{Reply, Request, Response}\n\npub fn main() {\n  use conn &lt;- result.then(glats.connect(&quot;localhost&quot;, 4222, []))\n\n  // Start a request handler actor that will call `ping_pong_handler`\n  // for every request received from NATS topic &quot;do.ping&quot;.\n  let assert Ok(_actor) =\n    handler.handle_request(conn, [], &quot;do.ping&quot;, [], ping_pong_handler)\n\n  process.sleep_forever()\n\n  Ok(Nil)\n}\n\npub fn ping_pong_handler(req: Request, state) {\n  // Got message: Hello\n  io.println(&quot;Got message: &quot; &lt;&gt; req.body)\n\n  // Reply with a message with the same headers and append to body.\n  Reply(\n    Response(\n      headers: req.headers,\n      reply_to: None,\n      body: req.body &lt;&gt; &quot; from glats!&quot;,\n    ),\n    state,\n  )\n}\n```\n\nThen in shell with `natscli`.\n\n```sh\n$ nats req do.ping &#39;Hello&#39;\n12:16:47 Sending request on &quot;do.ping&quot;\n12:16:47 Received with rtt 427.64µs\nHello from glats!\n```\n\n## Jetstream\n\n### Pull subscription\n\nPull consumers allow clients to request batches of messages on demand.\nFor best performance set the batch size to as high as possible, but keep\nin mind that the client should be able to process (and ack) the entire\nbatch in the `AckWait` time window of the consumer.\n\nIn the example below we request a batch of 50 messages with `NoWait` set.\nThis instructs NATS to immediately return a response, event when no pending\nmessages exist in the stream (in which case we get a message with status `404`).\nThat way we can exit the program when we&#39;re caught up.\n\nNormally you would request a batch with an expiry time (option `Expires(Int)`),\nin which case the request lasts for the time requested (and a message with status\n`408` is received when it expires).\n\nThis allows the client to control the flow of messages instead of NATS flooding\nit when there is high load as is possible with push consumers.\n\n[More info on pull consumers](https://www.byronruth.com/grokking-nats-consumers-part-3/)\n\nAll the logic in this example is handled for you in `glats/jetstream/handler.{handle_pull_consumer}`,\nsee [docs](https://hexdocs.pm/glats/glats/jetstream/handler.html).\n\n```gleam\nimport gleam/io\nimport gleam/string\nimport gleam/result\nimport gleam/option.{Some}\nimport gleam/otp/actor.{InitFailed}\nimport gleam/erlang/process.{Abnormal}\nimport glats.{ReceivedMessage}\nimport glats/jetstream\nimport glats/jetstream/stream\nimport glats/jetstream/consumer.{\n  AckExplicit, AckPolicy, Batch, BindStream, Description, InactiveThreshold,\n  NoWait, Subscription, With,\n}\n\nconst batch_size = 50\n\ntype State {\n  State(sub: Subscription, remaining: Int)\n}\n\npub fn main() {\n  use conn &lt;- result.then(glats.connect(&quot;localhost&quot;, 4222, []))\n\n  // Create a stream\n  let assert Ok(stream) =\n    stream.create(conn, &quot;mystream&quot;, [&quot;orders.&gt;&quot;, &quot;items.&gt;&quot;], [])\n\n  // Publish 3 messages to subjects contained in the stream\n  let assert Ok(Nil) = glats.publish(conn, &quot;orders.1&quot;, &quot;order_data&quot;, [])\n  let assert Ok(Nil) = glats.publish(conn, &quot;orders.2&quot;, &quot;order_data&quot;, [])\n  let assert Ok(Nil) = glats.publish(conn, &quot;items.1&quot;, &quot;item_data&quot;, [])\n\n  // Subscribe to subject in the stream using an ephemeral consumer\n  let subject = process.new_subject()\n  let assert Ok(sub) =\n    consumer.subscribe(\n      conn,\n      subject,\n      &quot;orders.*&quot;,\n      [\n        // Bind to stream created above\n        BindStream(stream.config.name),\n        // Set description for the ephemeral consumer\n        With(Description(&quot;An ephemeral consumer for subscription&quot;)),\n        // Set ack policy for the consumer\n        With(AckPolicy(AckExplicit)),\n        // Sets the inactive threshold of the ephemeral consumer\n        With(InactiveThreshold(60_000_000_000)),\n      ],\n    )\n    |&gt; io.debug\n\n  // Start loop\n  case request_and_loop(subject, State(sub: sub, remaining: 0)) {\n    Ok(Nil) -&gt; Ok(Nil)\n    Error(err) -&gt; {\n      io.println(&quot;got error: &quot; &lt;&gt; string.inspect(err))\n\n      Error(InitFailed(Abnormal(string.inspect(err))))\n    }\n  }\n}\n\n// Requests a batch of messages for the pull consumer and updates state\nfn request_and_loop(subject, state: State) {\n  case consumer.request_batch(state.sub, [Batch(batch_size), NoWait]) {\n    Ok(Nil) -&gt; loop(subject, State(..state, remaining: batch_size))\n    Error(_) -&gt; Error(Nil)\n  }\n}\n\n// Receives messages\nfn loop(subject, state: State) {\n  case process.receive(subject, 2000) {\n    // When request expires we get a message with status of `408`.\n    Ok(ReceivedMessage(status: Some(408), ..)) -&gt; {\n      io.println(&quot;request expired, requesting more&quot;)\n\n      request_and_loop(subject, state)\n    }\n\n    // When request is made with `NoWait` we get a message with status\n    // of `404` when no new messages exist.\n    Ok(ReceivedMessage(status: Some(404), ..)) -&gt; {\n      io.println(&quot;no new messages&quot;)\n\n      Ok(Nil)\n    }\n\n    // We got a new message!\n    Ok(ReceivedMessage(conn: conn, message: msg, ..)) -&gt; {\n      // Print message\n      io.debug(msg)\n\n      // Acknowledge message\n      let assert Ok(_) = jetstream.ack(conn, msg)\n\n      // Keep track of remaining messages in the request\n      case state.remaining &lt;= 1 {\n        // Request more\n        True -&gt; request_and_loop(subject, state)\n        // Decrement the counter\n        False -&gt; loop(subject, State(..state, remaining: state.remaining - 1))\n      }\n    }\n\n    // An error!\n    Error(err) -&gt; Error(err)\n  }\n}\n```\n\n### Push subscription\n\nWith push consumers messages are automatically published on a\nspecified subject.\n\nThis example is considerably simpler than the pull consumer above\nbut keep in mind that with push consumers the flow of messages is\nharder to control.\n\n[More info on push consumers](https://www.byronruth.com/grokking-nats-consumers-part-1/)\n\n```gleam\nimport gleam/io\nimport gleam/result\nimport gleam/erlang/process.{Subject}\nimport glats.{ReceivedMessage, SubscriptionMessage}\nimport glats/jetstream\nimport glats/jetstream/stream\nimport glats/jetstream/consumer.{\n  AckExplicit, AckPolicy, BindStream, DeliverSubject, Description,\n  InactiveThreshold, With,\n}\n\npub fn main() {\n  use conn &lt;- result.then(glats.connect(&quot;localhost&quot;, 4222, []))\n\n  // Create a stream\n  let assert Ok(stream) =\n    stream.create(conn, &quot;mystream&quot;, [&quot;orders.&gt;&quot;, &quot;items.&gt;&quot;], [])\n\n  // Publish 3 messages to subjects contained in the stream\n  let assert Ok(Nil) = glats.publish(conn, &quot;orders.1&quot;, &quot;order_data&quot;, [])\n  let assert Ok(Nil) = glats.publish(conn, &quot;orders.2&quot;, &quot;order_data&quot;, [])\n  let assert Ok(Nil) = glats.publish(conn, &quot;items.1&quot;, &quot;item_data&quot;, [])\n\n  // Generate a random inbox topic for the push consumer&#39;s delivery topic\n  let inbox = glats.new_inbox()\n\n  // Subscribe to subject in the stream using an ephemeral consumer\n  let subject = process.new_subject()\n  let assert Ok(_sub) =\n    consumer.subscribe(\n      conn,\n      subject,\n      &quot;orders.*&quot;,\n      [\n        // Bind to stream created above\n        BindStream(stream.config.name),\n        // Make it a push consumer\n        With(DeliverSubject(inbox)),\n        // Set description for the ephemeral consumer\n        With(Description(&quot;An ephemeral consumer for subscription&quot;)),\n        // Set ack policy for the consumer\n        With(AckPolicy(AckExplicit)),\n        // Sets the inactive threshold of the ephemeral consumer\n        With(InactiveThreshold(60_000_000_000)),\n      ],\n    )\n    |&gt; io.debug\n\n  // Start loop\n  loop(subject)\n}\n\nfn loop(subject: Subject(SubscriptionMessage)) {\n  case process.receive(subject, 2000) {\n    // New message received\n    Ok(ReceivedMessage(conn: conn, message: msg, ..)) -&gt; {\n      // Print message\n      io.debug(msg)\n\n      // Acknowledge message\n      let assert Ok(Nil) = jetstream.ack(conn, msg)\n\n      // Run loop again\n      loop(subject)\n    }\n\n    // Error!\n    Error(Nil) -&gt; {\n      io.println(&quot;no new message in 2 seconds&quot;)\n\n      Ok(Nil)\n    }\n  }\n}\n```\n\n## Installation\n\n```sh\ngleam add glats\n```\n\nand its documentation can be found at &lt;https://hexdocs.pm/glats&gt;.\n","url":"index.html"},{"doc":"glats","title":"Connection","content":"pub type Connection =\n  process.Subject(ConnectionMessage)\n\n\nSynonyms:\nglats.Connection\nglats Connection","url":"glats.html#Connection"},{"doc":"glats","title":"ConnectionMessage","content":"pub opaque type ConnectionMessage\n Message sent to a NATS connection process.\n\nSynonyms:\nglats.ConnectionMessage\nglats ConnectionMessage","url":"glats.html#ConnectionMessage"},{"doc":"glats","title":"ConnectionOption","content":"pub type ConnectionOption {\n  UserPass(String, String)\n  Token(String)\n  NKeySeed(String)\n  JWT(String)\n  CACert(String)\n  ClientCert(String, String)\n  InboxPrefix(String)\n  ConnectionTimeout(Int)\n  EnableNoResponders\n}\n Options that can be passed to `connect`.\n\nUserPass(String, String)\n Set a username and password for authentication.\n\nToken(String)\n Set a token for authentication.\n\nNKeySeed(String)\n Set an NKey seed for authentication.\n\nJWT(String)\n Set a JWT for authentication.\n\nCACert(String)\n Set a CA cert for the server connection.\n\nClientCert(String, String)\n Set client certificate key pair for authentication.\n\nInboxPrefix(String)\n Set a custom inbox prefix used by the connection.\n\nConnectionTimeout(Int)\n Set a connection timeout in milliseconds.\n\nEnableNoResponders\n Enable the no responders behavior.\n\nSynonyms:\nglats.ConnectionOption\nglats ConnectionOption","url":"glats.html#ConnectionOption"},{"doc":"glats","title":"Error","content":"pub type Error {\n  NoReplyTopic\n  Timeout\n  NoResponders\n  Unexpected\n}\n Errors that can be returned by the server.\n\nNoReplyTopic\n\n\nTimeout\n\n\nNoResponders\n\n\nUnexpected\n\n\nSynonyms:\nglats.Error\nglats Error","url":"glats.html#Error"},{"doc":"glats","title":"Message","content":"pub type Message {\n  Message(\n    topic: String,\n    headers: dict.Dict(String, String),\n    reply_to: option.Option(String),\n    body: String,\n  )\n}\n A single message that can be received from or sent to NATS.\n\nMessage(\n  topic: String,\n  headers: dict.Dict(String, String),\n  reply_to: option.Option(String),\n  body: String,\n)\n\n\nSynonyms:\nglats.Message\nglats Message","url":"glats.html#Message"},{"doc":"glats","title":"PublishOption","content":"pub type PublishOption {\n  ReplyTo(String)\n  Headers(List(#(String, String)))\n}\n Option that can be specified when publishing a message.\n\nReplyTo(String)\n Provide a reply topic with the message that the receiver\n can send data back to.\n\nHeaders(List(#(String, String)))\n Headers to add to the message.\n\nSynonyms:\nglats.PublishOption\nglats PublishOption","url":"glats.html#PublishOption"},{"doc":"glats","title":"ServerInfo","content":"pub type ServerInfo {\n  ServerInfo(\n    server_id: String,\n    server_name: String,\n    version: String,\n    go: String,\n    host: String,\n    port: Int,\n    headers: Bool,\n    max_payload: Int,\n    proto: Int,\n    jetstream: Bool,\n    auth_required: option.Option(Bool),\n  )\n}\n Server info returned by the NATS server.\n\nServerInfo(\n  server_id: String,\n  server_name: String,\n  version: String,\n  go: String,\n  host: String,\n  port: Int,\n  headers: Bool,\n  max_payload: Int,\n  proto: Int,\n  jetstream: Bool,\n  auth_required: option.Option(Bool),\n)\n\n\nSynonyms:\nglats.ServerInfo\nglats ServerInfo","url":"glats.html#ServerInfo"},{"doc":"glats","title":"SubscribeOption","content":"pub type SubscribeOption {\n  QueueGroup(String)\n}\n Option that can be specified when subscribing to a topic.\n\nQueueGroup(String)\n Subscribe to a topic as part of a queue group.\n\nSynonyms:\nglats.SubscribeOption\nglats SubscribeOption","url":"glats.html#SubscribeOption"},{"doc":"glats","title":"SubscriptionMessage","content":"pub type SubscriptionMessage {\n  ReceivedMessage(\n    conn: Connection,\n    sid: Int,\n    status: option.Option(Int),\n    message: Message,\n  )\n}\n Message received by a subscribing subject.\n\nReceivedMessage(\n  conn: Connection,\n  sid: Int,\n  status: option.Option(Int),\n  message: Message,\n)\n\n\nSynonyms:\nglats.SubscriptionMessage\nglats SubscriptionMessage","url":"glats.html#SubscriptionMessage"},{"doc":"glats","title":"active_subscriptions","content":"pub fn active_subscriptions(\n  conn: Subject(ConnectionMessage),\n) -&gt; Result(Int, Error)\n Returns the number of active subscriptions for the connection.\n\nSynonyms:\nglats.active_subscriptions\nglats active_subscriptions","url":"glats.html#active_subscriptions"},{"doc":"glats","title":"connect","content":"pub fn connect(\n  host: String,\n  port: Int,\n  opts: List(ConnectionOption),\n) -&gt; Result(Subject(ConnectionMessage), StartError)\n Starts an actor that handles a connection to NATS using the provided\n settings.\n\n ## Example\n\n \n connect(\n   &quot;localhost&quot;,\n   4222,\n   [\n     CACert(&quot;/tmp/nats/ca.crt&quot;),\n     InboxPrefix(&quot;_INBOX.custom.prefix.&quot;),\n   ],\n )\n \n\nSynonyms:\nglats.connect\nglats connect","url":"glats.html#connect"},{"doc":"glats","title":"new_inbox","content":"pub fn new_inbox() -&gt; String\n Returns a new random inbox.\n\nSynonyms:\nglats.new_inbox\nglats new_inbox","url":"glats.html#new_inbox"},{"doc":"glats","title":"publish","content":"pub fn publish(\n  conn: Subject(ConnectionMessage),\n  topic: String,\n  body: String,\n  opts: List(PublishOption),\n) -&gt; Result(Nil, Error)\n Publishes a single message to NATS on a provided topic.\n\nSynonyms:\nglats.publish\nglats publish","url":"glats.html#publish"},{"doc":"glats","title":"publish_message","content":"pub fn publish_message(\n  conn: Subject(ConnectionMessage),\n  message: Message,\n) -&gt; Result(Nil, Error)\n Publishes a single message to NATS using the data from a provided `Message`\n record.\n\nSynonyms:\nglats.publish_message\nglats publish_message","url":"glats.html#publish_message"},{"doc":"glats","title":"request","content":"pub fn request(\n  conn: Subject(ConnectionMessage),\n  topic: String,\n  body: String,\n  opts: List(PublishOption),\n  timeout: Int,\n) -&gt; Result(Message, Error)\n Sends a request and listens for a response synchronously.\n When connection is established with option `EnableNoResponders`,\n `Error(NoResponders)` will be returned immediately if no subscriber\n exists for the topic.\n\n See [request-reply pattern docs.](https://docs.nats.io/nats-concepts/core-nats/reqreply)\n\n To handle a request from NATS see `handler.handle_request`.\n\nSynonyms:\nglats.request\nglats request","url":"glats.html#request"},{"doc":"glats","title":"respond","content":"pub fn respond(\n  conn: Subject(ConnectionMessage),\n  message: Message,\n  body: String,\n  opts: List(PublishOption),\n) -&gt; Result(Nil, Error)\n Sends a respond to a Message&#39;s reply_to topic.\n\nSynonyms:\nglats.respond\nglats respond","url":"glats.html#respond"},{"doc":"glats","title":"server_info","content":"pub fn server_info(\n  conn: Subject(ConnectionMessage),\n) -&gt; Result(ServerInfo, Error)\n Returns server info provided by the connected NATS server.\n\nSynonyms:\nglats.server_info\nglats server_info","url":"glats.html#server_info"},{"doc":"glats","title":"subscribe","content":"pub fn subscribe(\n  conn: Subject(ConnectionMessage),\n  subscriber: Subject(SubscriptionMessage),\n  topic: String,\n  opts: List(SubscribeOption),\n) -&gt; Result(Int, Error)\n Subscribes to a NATS topic that can be received on the\n provided OTP subject.\n\n To subscribe as a member of a queue group, add option\n `QueueGroup(String)` to list of options as the last\n parameter.\n\n \n subscribe(conn, subject, &quot;my.topic&quot;, [QueueGroup(&quot;my-group&quot;)])\n \n\n See [Queue Groups docs.](https://docs.nats.io/nats-concepts/core-nats/queue)\n\nSynonyms:\nglats.subscribe\nglats subscribe","url":"glats.html#subscribe"},{"doc":"glats","title":"unsubscribe","content":"pub fn unsubscribe(\n  conn: Subject(ConnectionMessage),\n  sid: Int,\n) -&gt; Result(Nil, Error)\n Unsubscribe from a subscription by providing the subscription ID.\n\nSynonyms:\nglats.unsubscribe\nglats unsubscribe","url":"glats.html#unsubscribe"},{"doc":"glats","title":"glats","content":"","url":"glats.html"},{"doc":"glats/handler","title":"Request","content":"pub type Request {\n  Request(headers: dict.Dict(String, String), body: String)\n}\n The message data received from the request handler&#39;s topic.\n\nRequest(headers: dict.Dict(String, String), body: String)\n\n\nSynonyms:\nglats/handler.Request\nglats/handler Request","url":"glats/handler.html#Request"},{"doc":"glats/handler","title":"RequestHandler","content":"pub type RequestHandler(a) =\n  fn(Request, a) -&gt; RequestOutcome(a)\n The request handling callback that should be passed to the request handler.\n This will be called for every request received.\n\nSynonyms:\nglats/handler.RequestHandler\nglats/handler RequestHandler","url":"glats/handler.html#RequestHandler"},{"doc":"glats/handler","title":"RequestOutcome","content":"pub type RequestOutcome(a) {\n  Reply(response: Response, state: a)\n  Stop(process.ExitReason)\n}\n Next step for the request handler to do.\n\nReply(response: Response, state: a)\n The request handler will reply to the requester with the\n response and save the state.\n\n\nStop(process.ExitReason)\n The request handler will stop with provided exit reason.\n\n\nSynonyms:\nglats/handler.RequestOutcome\nglats/handler RequestOutcome","url":"glats/handler.html#RequestOutcome"},{"doc":"glats/handler","title":"Response","content":"pub type Response {\n  Response(\n    headers: dict.Dict(String, String),\n    reply_to: option.Option(String),\n    body: String,\n  )\n}\n The message data that should be replied to the requester.\n\nResponse(\n  headers: dict.Dict(String, String),\n  reply_to: option.Option(String),\n  body: String,\n)\n\n\nSynonyms:\nglats/handler.Response\nglats/handler Response","url":"glats/handler.html#Response"},{"doc":"glats/handler","title":"handle_request","content":"pub fn handle_request(\n  conn: Subject(ConnectionMessage),\n  state: a,\n  topic: String,\n  opts: List(SubscribeOption),\n  handler: fn(Request, a) -&gt; RequestOutcome(a),\n) -&gt; Result(Subject(SubscriptionMessage), StartError)\n Starts an actor that subscribes to the desired NATS topic and calls the\n provided request handler with the request data and replies to NATS with\n the returned message data from the request handler.\n\nSynonyms:\nglats/handler.handle_request\nglats/handler handle_request","url":"glats/handler.html#handle_request"},{"doc":"glats/handler","title":"glats/handler","content":" Request handler will handle receiving messages from a subscription,\n pass the data to a callback and then reply to NATS with the returned\n data.\n\n ## Example\n\n ```gleam\n import gleam/option.{None}\n import gleam/result\n import gleam/erlang/process\n import glats\n import glats/handler.{Reply, Request, Response}\n \n pub fn main() {\n   use conn &lt;- result.then(glats.connect(&quot;localhost&quot;, 4222, []))\n \n   // Start a request handler actor that will call `ping_pong_handler` for\n   // every request received from NATS topic &quot;do.ping&quot;.\n   assert Ok(_actor) =\n     handler.handle_request(conn, [], &quot;do.ping&quot;, None, ping_pong_handler)\n \n   process.sleep_forever()\n \n   Ok(Nil)\n }\n \n pub fn ping_pong_handler(req: Request, state) {\n   // Got message: Hello\n   io.println(&quot;Got message: &quot; &lt;&gt; req.body)\n\n   // Reply with a message with the same headers and append to body.\n   Reply(\n     Response(\n       headers: req.headers,\n       reply_to: None,\n       body: req.body &lt;&gt; &quot; from glats!&quot;,\n     ),\n     state,\n   )\n }\n ```\n\n Then in a shell with `natscli`.\n\n ```sh\n $ nats req do.ping &#39;Hello&#39;\n 12:16:47 Sending request on &quot;do.ping&quot;\n 12:16:47 Received with rtt 427.64µs\n Hello from glats!\n ```\n","url":"glats/handler.html"},{"doc":"glats/jetstream","title":"JetstreamError","content":"pub type JetstreamError {\n  JetstreamNotEnabledForAccount(String)\n  JetstreamNotEnabled(String)\n  InsufficientResources(String)\n  InvalidStreamConfig(String)\n  StreamNameInSubjectDoesNotMatch(String)\n  StreamNameInUse(String)\n  StreamNotFound(String)\n  StreamPurgeNotAllowed(String)\n  NoMessageFound(String)\n  ConsumerNotFound(String)\n  ConsumerNameExists(String)\n  ConsumerAlreadyExists(String)\n  WrongLastSequence(String)\n  BadRequest(String)\n  Unknown(Int, String)\n  DecodeError(String)\n  Timeout\n  NoResponders\n  PullConsumerRequired(String)\n}\n Errors that can be returned when working with Jetstream.\n\nJetstreamNotEnabledForAccount(String)\n\n\nJetstreamNotEnabled(String)\n\n\nInsufficientResources(String)\n\n\nInvalidStreamConfig(String)\n\n\nStreamNameInSubjectDoesNotMatch(String)\n\n\nStreamNameInUse(String)\n\n\nStreamNotFound(String)\n\n\nStreamPurgeNotAllowed(String)\n\n\nNoMessageFound(String)\n\n\nConsumerNotFound(String)\n\n\nConsumerNameExists(String)\n\n\nConsumerAlreadyExists(String)\n\n\nWrongLastSequence(String)\n\n\nBadRequest(String)\n\n\nUnknown(Int, String)\n\n\nDecodeError(String)\n\n\nTimeout\n\n\nNoResponders\n\n\nPullConsumerRequired(String)\n\n\nSynonyms:\nglats/jetstream.JetstreamError\nglats/jetstream JetstreamError","url":"glats/jetstream.html#JetstreamError"},{"doc":"glats/jetstream","title":"StorageType","content":"pub type StorageType {\n  FileStorage\n  MemoryStorage\n}\n Sets the storage type in a stream.\n\nFileStorage\n\n\nMemoryStorage\n\n\nSynonyms:\nglats/jetstream.StorageType\nglats/jetstream StorageType","url":"glats/jetstream.html#StorageType"},{"doc":"glats/jetstream","title":"ack","content":"pub fn ack(\n  conn: Subject(ConnectionMessage),\n  message: Message,\n) -&gt; Result(Nil, Error)\n Sends an acknowledgement for a message.\n\nSynonyms:\nglats/jetstream.ack\nglats/jetstream ack","url":"glats/jetstream.html#ack"},{"doc":"glats/jetstream","title":"nack","content":"pub fn nack(\n  conn: Subject(ConnectionMessage),\n  message: Message,\n) -&gt; Result(Nil, Error)\n Sends a negative acknowledgement for a message.\n\n Delivery will be retried until ack&#39;d or term&#39;d.\n\nSynonyms:\nglats/jetstream.nack\nglats/jetstream nack","url":"glats/jetstream.html#nack"},{"doc":"glats/jetstream","title":"nack_delay","content":"pub fn nack_delay(\n  conn: Subject(ConnectionMessage),\n  message: Message,\n  with delay: Int,\n) -&gt; Result(Nil, Error)\n Sends a negative acknowledgement for a message and delays\n redelivery of the message. The unit is in nanoseconds.\n\nSynonyms:\nglats/jetstream.nack_delay\nglats/jetstream nack_delay","url":"glats/jetstream.html#nack_delay"},{"doc":"glats/jetstream","title":"term","content":"pub fn term(\n  conn: Subject(ConnectionMessage),\n  message: Message,\n) -&gt; Result(Nil, Error)\n Sends a term acknowledgement for a message.\n\n Instructs the server to stop redelivery of a message without acknowledging\n it as successfully processed.\n\nSynonyms:\nglats/jetstream.term\nglats/jetstream term","url":"glats/jetstream.html#term"},{"doc":"glats/jetstream","title":"glats/jetstream","content":"","url":"glats/jetstream.html"},{"doc":"glats/jetstream/stream","title":"AccessMethod","content":"pub type AccessMethod {\n  SequenceID(Int)\n  LastBySubject(String)\n}\n Access method type for a get message request to Jetstream.\n\n See `get_message`.\n\nSequenceID(Int)\n Used to get a message from a stream by sequence ID.\n\n\nLastBySubject(String)\n Used to get newest message from a stream that matches\n a subject.\n\n\nSynonyms:\nglats/jetstream/stream.AccessMethod\nglats/jetstream/stream AccessMethod","url":"glats/jetstream/stream.html#AccessMethod"},{"doc":"glats/jetstream/stream","title":"DiscardPolicy","content":"pub type DiscardPolicy {\n  DiscardOld\n  DiscardNew\n}\n Used to set the discard policy of a stream.\n\nDiscardOld\n This policy will delete the oldest messages in order\n to maintain the limit. For example, if `MaxAge` is set\n to one minute, the server will automatically delete\n messages older than one minute with this policy.\n\nDiscardNew\n This policy will reject new messages from being\n appended to the stream if it would exceed one of the\n limits. An extension to this policy is\n `DiscardNewPerSubject` which will apply this policy\n on a per-subject basis within the stream.\n\nSynonyms:\nglats/jetstream/stream.DiscardPolicy\nglats/jetstream/stream DiscardPolicy","url":"glats/jetstream/stream.html#DiscardPolicy"},{"doc":"glats/jetstream/stream","title":"RetentionPolicy","content":"pub type RetentionPolicy {\n  LimitsPolicy\n  InterestPolicy\n  WorkQueuePolicy\n}\n Used to set the retention policy of a stream.\n\nLimitsPolicy\n Retention based on the various limits that are set\n including: `MaxMessages`, `MaxBytes`, `MaxAge`, and\n `MaxMessagesPerSubject`. If any of these limits are\n set, whichever limit is hit first will cause the\n automatic deletion of the respective message(s).\n\nInterestPolicy\n Retention based on the consumer interest in the\n stream and messages. The base case is that there\n are zero consumers defined for a stream. If messages\n are published to the stream, they will be immediately\n deleted so there is no _interest_.\n\nWorkQueuePolicy\n Retention with the typical behavior of a FIFO queue.\n Each message can be consumed only once. This is\n enforced by only allowing _one_ consumer to be created\n for a work-queue stream.\n\nSynonyms:\nglats/jetstream/stream.RetentionPolicy\nglats/jetstream/stream RetentionPolicy","url":"glats/jetstream/stream.html#RetentionPolicy"},{"doc":"glats/jetstream/stream","title":"StreamConfig","content":"pub type StreamConfig {\n  StreamConfig(\n    name: String,\n    subjects: List(String),\n    retention: option.Option(String),\n    max_consumers: option.Option(Int),\n    max_msgs: option.Option(Int),\n    max_bytes: option.Option(Int),\n    max_age: option.Option(Int),\n    max_msgs_per_subject: option.Option(Int),\n    max_msg_size: option.Option(Int),\n    discard: option.Option(String),\n    storage: option.Option(String),\n    num_replicas: option.Option(Int),\n    duplicate_window: option.Option(Int),\n    allow_direct: option.Option(Bool),\n    mirror_direct: option.Option(Bool),\n    sealed: option.Option(Bool),\n    deny_delete: option.Option(Bool),\n    deny_purge: option.Option(Bool),\n    allow_rollup_hdrs: option.Option(Bool),\n  )\n}\n Available config options for a single stream.\n\nStreamConfig(\n  name: String,\n  subjects: List(String),\n  retention: option.Option(String),\n  max_consumers: option.Option(Int),\n  max_msgs: option.Option(Int),\n  max_bytes: option.Option(Int),\n  max_age: option.Option(Int),\n  max_msgs_per_subject: option.Option(Int),\n  max_msg_size: option.Option(Int),\n  discard: option.Option(String),\n  storage: option.Option(String),\n  num_replicas: option.Option(Int),\n  duplicate_window: option.Option(Int),\n  allow_direct: option.Option(Bool),\n  mirror_direct: option.Option(Bool),\n  sealed: option.Option(Bool),\n  deny_delete: option.Option(Bool),\n  deny_purge: option.Option(Bool),\n  allow_rollup_hdrs: option.Option(Bool),\n)\n\n\nSynonyms:\nglats/jetstream/stream.StreamConfig\nglats/jetstream/stream StreamConfig","url":"glats/jetstream/stream.html#StreamConfig"},{"doc":"glats/jetstream/stream","title":"StreamInfo","content":"pub type StreamInfo {\n  StreamInfo(\n    created: String,\n    config: StreamConfig,\n    state: StreamState,\n  )\n}\n Info about stream returned from `info` function.\n\nStreamInfo(\n  created: String,\n  config: StreamConfig,\n  state: StreamState,\n)\n\n\nSynonyms:\nglats/jetstream/stream.StreamInfo\nglats/jetstream/stream StreamInfo","url":"glats/jetstream/stream.html#StreamInfo"},{"doc":"glats/jetstream/stream","title":"StreamMessage","content":"pub type StreamMessage {\n  StreamMessage(\n    sequence: Int,\n    time: String,\n    message: glats.Message,\n  )\n}\n\nStreamMessage(\n  sequence: Int,\n  time: String,\n  message: glats.Message,\n)\n\n\nSynonyms:\nglats/jetstream/stream.StreamMessage\nglats/jetstream/stream StreamMessage","url":"glats/jetstream/stream.html#StreamMessage"},{"doc":"glats/jetstream/stream","title":"StreamOption","content":"pub type StreamOption {\n  Description(String)\n  Retention(RetentionPolicy)\n  MaxConsumers(Int)\n  MaxMessages(Int)\n  MaxBytes(Int)\n  MaxAge(Int)\n  MaxMessagesPerSubject(Int)\n  MaxMessageSize(Int)\n  Discard(DiscardPolicy)\n  Storage(jetstream.StorageType)\n  NumReplicas(Int)\n  DuplicateWindow(Int)\n  AllowDirect(Bool)\n  MirrorDirect(Bool)\n  DenyDelete(Bool)\n  DenyPurge(Bool)\n  AllowRollup(Bool)\n  DiscardNewPerSubject(Bool)\n}\n Available options to set during stream creation and update.\n\nDescription(String)\n A verbose description of the stream.\n\nRetention(RetentionPolicy)\n Declares the retention policy for the stream.\n\n See: https://docs.nats.io/nats-concepts/jetstream/streams#retentionpolicy\n\nMaxConsumers(Int)\n How many Consumers can be defined for a given Stream,\n -1 for unlimited.\n\nMaxMessages(Int)\n How many messages may be in a Stream. Adheres to Discard\n Policy, removing oldest or refusing new messages if the\n Stream exceeds this number of messages.\n\nMaxBytes(Int)\n How many bytes the Stream may contain. Adheres to\n Discard Policy, removing oldest or refusing new messages\n if the Stream exceeds this size\n\nMaxAge(Int)\n Maximum age of any message in the Stream, expressed\n in nanoseconds.\n\nMaxMessagesPerSubject(Int)\n Limits how many messages in the stream to retain per subject.\n\nMaxMessageSize(Int)\n The largest message that will be accepted by the Stream\n\nDiscard(DiscardPolicy)\n The behavior of discarding messages when any streams&#39;\n limits have been reached.\n\n See: https://docs.nats.io/nats-concepts/jetstream/streams#discardpolicy\n\nStorage(jetstream.StorageType)\n The storage type for stream data.\n\nNumReplicas(Int)\n How many replicas to keep for each message in a\n clustered JetStream, maximum 5.\n\nDuplicateWindow(Int)\n The window within which to track duplicate messages,\n expressed in nanoseconds.\n\nAllowDirect(Bool)\n If true, and the stream has more than one replica, each\n replica will respond to direct get requests for individual\n messages, not only the leader.\n\nMirrorDirect(Bool)\n If true, and the stream is a mirror, the mirror will\n participate in a serving direct get requests for individual\n messages from origin stream.\n\nDenyDelete(Bool)\n Restricts the ability to delete messages from a stream\n via the API.\n\nDenyPurge(Bool)\n Restricts the ability to purge messages from a stream\n via the API.\n\nAllowRollup(Bool)\n Allows the use of the Nats-Rollup header to replace all\n contents of a stream, or subject in a stream, with a\n single new message.\n\nDiscardNewPerSubject(Bool)\n If `True`, applies discard new semantics on a per subject\n basis. Requires `DiscardPolicy` to be `DiscardNew` and\n the `MaxMessagesPerSubject` to be set.\n\nSynonyms:\nglats/jetstream/stream.StreamOption\nglats/jetstream/stream StreamOption","url":"glats/jetstream/stream.html#StreamOption"},{"doc":"glats/jetstream/stream","title":"StreamState","content":"pub type StreamState {\n  StreamState(\n    messages: Int,\n    bytes: Int,\n    first_seq: Int,\n    first_ts: String,\n    last_seq: Int,\n    last_ts: String,\n    consumer_count: Int,\n  )\n}\n Stream state.\n\nStreamState(\n  messages: Int,\n  bytes: Int,\n  first_seq: Int,\n  first_ts: String,\n  last_seq: Int,\n  last_ts: String,\n  consumer_count: Int,\n)\n\n\nSynonyms:\nglats/jetstream/stream.StreamState\nglats/jetstream/stream StreamState","url":"glats/jetstream/stream.html#StreamState"},{"doc":"glats/jetstream/stream","title":"create","content":"pub fn create(\n  conn: Subject(ConnectionMessage),\n  name: String,\n  subjects: List(String),\n  opts: List(StreamOption),\n) -&gt; Result(StreamInfo, JetstreamError)\n Creates a new stream.\n\n Calling this when a stream by the same already exists will run successfully\n when no mutable field in the config is different.\n\n ## Example\n\n \n create(\n   conn,\n   &quot;samplestream&quot;,\n   [&quot;sample.subject.&gt;&quot;],\n   [\n     Storage(MemoryStorage),\n     Retention(WorkQueuePolicy),\n   ],\n )\n \nSynonyms:\nglats/jetstream/stream.create\nglats/jetstream/stream create","url":"glats/jetstream/stream.html#create"},{"doc":"glats/jetstream/stream","title":"delete","content":"pub fn delete(\n  conn: Subject(ConnectionMessage),\n  name: String,\n) -&gt; Result(Nil, JetstreamError)\n Deletes a stream.\n\nSynonyms:\nglats/jetstream/stream.delete\nglats/jetstream/stream delete","url":"glats/jetstream/stream.html#delete"},{"doc":"glats/jetstream/stream","title":"find_stream_name_by_subject","content":"pub fn find_stream_name_by_subject(\n  conn: Subject(ConnectionMessage),\n  subject: String,\n) -&gt; Result(String, JetstreamError)\n Tries to find a stream name by subject.\n\nSynonyms:\nglats/jetstream/stream.find_stream_name_by_subject\nglats/jetstream/stream find_stream_name_by_subject","url":"glats/jetstream/stream.html#find_stream_name_by_subject"},{"doc":"glats/jetstream/stream","title":"get_message","content":"pub fn get_message(\n  conn: Subject(ConnectionMessage),\n  stream: String,\n  method: AccessMethod,\n) -&gt; Result(StreamMessage, JetstreamError)\n Directly fetches a message from a stream either by sequence ID\n (by passing `SequenceID(Int)`) or by subject (by passing\n `LastBySubject(String)`).\n\n A subject can have wildcards (e.g. `orders.*.item.&gt;`), please\n refer to [Subject-Based messaging docs](https://docs.nats.io/nats-concepts/subjects)\n for more info.\n\n Keep in mind that `allow_direct` has to be enabled in stream config\n for this to work.\n\nSynonyms:\nglats/jetstream/stream.get_message\nglats/jetstream/stream get_message","url":"glats/jetstream/stream.html#get_message"},{"doc":"glats/jetstream/stream","title":"info","content":"pub fn info(\n  conn: Subject(ConnectionMessage),\n  name: String,\n) -&gt; Result(StreamInfo, JetstreamError)\n Get info about a stream by name.\n\nSynonyms:\nglats/jetstream/stream.info\nglats/jetstream/stream info","url":"glats/jetstream/stream.html#info"},{"doc":"glats/jetstream/stream","title":"purge","content":"pub fn purge(\n  conn: Subject(ConnectionMessage),\n  name: String,\n) -&gt; Result(Int, JetstreamError)\n Purges all of the data in a Stream, leaves the Stream.\n\nSynonyms:\nglats/jetstream/stream.purge\nglats/jetstream/stream purge","url":"glats/jetstream/stream.html#purge"},{"doc":"glats/jetstream/stream","title":"update","content":"pub fn update(\n  conn: Subject(ConnectionMessage),\n  name: String,\n  opts: List(StreamOption),\n) -&gt; Result(StreamInfo, JetstreamError)\n Updates the config of a stream.\n\nSynonyms:\nglats/jetstream/stream.update\nglats/jetstream/stream update","url":"glats/jetstream/stream.html#update"},{"doc":"glats/jetstream/stream","title":"glats/jetstream/stream","content":"","url":"glats/jetstream/stream.html"},{"doc":"glats/jetstream/consumer","title":"AckPolicy","content":"pub type AckPolicy {\n  AckExplicit\n  AckNone\n  AckAll\n}\n The requirement of client acknowledgements.\n\nAckExplicit\n The default policy. It means that each individual message must be\n acknowledged. It is recommended to use this mode, as it provides\n the most reliability and functionality.\n\nAckNone\n You do not have to ack any messages, the server will assume ack on\n delivery.\n\nAckAll\n If you receive a series of messages, you only have to ack the last\n one you received. All the previous messages received are automatically\n acknowledged at the same time.\n\nSynonyms:\nglats/jetstream/consumer.AckPolicy\nglats/jetstream/consumer AckPolicy","url":"glats/jetstream/consumer.html#AckPolicy"},{"doc":"glats/jetstream/consumer","title":"ConsumerConfig","content":"pub type ConsumerConfig {\n  ConsumerConfig(\n    durable_name: option.Option(String),\n    description: option.Option(String),\n    filter_subject: option.Option(String),\n    ack_policy: AckPolicy,\n    ack_wait: option.Option(Int),\n    deliver_policy: DeliverPolicy,\n    inactive_threshold: option.Option(Int),\n    max_ack_pending: option.Option(Int),\n    max_deliver: option.Option(Int),\n    replay_policy: ReplayPolicy,\n    num_replicas: option.Option(Int),\n    sample_freq: option.Option(String),\n    deliver_subject: option.Option(String),\n    deliver_group: option.Option(String),\n    headers_only: option.Option(Bool),\n  )\n}\n Avaialble config options for a single consumer.\n\nConsumerConfig(\n  durable_name: option.Option(String),\n  description: option.Option(String),\n  filter_subject: option.Option(String),\n  ack_policy: AckPolicy,\n  ack_wait: option.Option(Int),\n  deliver_policy: DeliverPolicy,\n  inactive_threshold: option.Option(Int),\n  max_ack_pending: option.Option(Int),\n  max_deliver: option.Option(Int),\n  replay_policy: ReplayPolicy,\n  num_replicas: option.Option(Int),\n  sample_freq: option.Option(String),\n  deliver_subject: option.Option(String),\n  deliver_group: option.Option(String),\n  headers_only: option.Option(Bool),\n)\n\n\nSynonyms:\nglats/jetstream/consumer.ConsumerConfig\nglats/jetstream/consumer ConsumerConfig","url":"glats/jetstream/consumer.html#ConsumerConfig"},{"doc":"glats/jetstream/consumer","title":"ConsumerInfo","content":"pub type ConsumerInfo {\n  ConsumerInfo(\n    stream: String,\n    name: String,\n    created: String,\n    config: ConsumerConfig,\n    delivered: SequenceInfo,\n    ack_floor: SequenceInfo,\n    num_ack_pending: Int,\n    num_redelivered: Int,\n    num_waiting: Int,\n    num_pending: Int,\n  )\n}\n\nConsumerInfo(\n  stream: String,\n  name: String,\n  created: String,\n  config: ConsumerConfig,\n  delivered: SequenceInfo,\n  ack_floor: SequenceInfo,\n  num_ack_pending: Int,\n  num_redelivered: Int,\n  num_waiting: Int,\n  num_pending: Int,\n)\n\n\nSynonyms:\nglats/jetstream/consumer.ConsumerInfo\nglats/jetstream/consumer ConsumerInfo","url":"glats/jetstream/consumer.html#ConsumerInfo"},{"doc":"glats/jetstream/consumer","title":"ConsumerOption","content":"pub type ConsumerOption {\n  DurableName(String)\n  Description(String)\n  FilterSubject(String)\n  AckPolicy(AckPolicy)\n  AckWait(Int)\n  DeliverPolicy(DeliverPolicy)\n  InactiveThreshold(Int)\n  MaxAckPending(Int)\n  MaxDeliver(Int)\n  ReplayPolicy(ReplayPolicy)\n  NumReplicas(Int)\n  SampleFrequency(String)\n  MaxWaiting(Int)\n  MaxRequestExpires(Int)\n  MaxRequestBatch(Int)\n  MaxRequestMaxBytes(Int)\n  DeliverSubject(String)\n  DeliverGroup(String)\n  HeadersOnly\n}\n Available options to set during consumer creation and update.\n\nDurableName(String)\n If set, clients can have subscriptions bind to the consumer and\n resume until the consumer is explicitly deleted. A durable name\n cannot contain whitespace, `.`, `*`, `&gt;`, path separators\n (forward or backwards slash), and non-printable characters.\n\nDescription(String)\n A description of the consumer. This can be particularly useful\n for ephemeral consumers to indicate their purpose since the durable\n name cannot be provided.\n\nFilterSubject(String)\n An overlapping subject with the subjects bound to the stream\n which will filter the set of messages received by the consumer.\n\nAckPolicy(AckPolicy)\n The requirement of client acknowledgements, either `AckExplicit`,\n `AckNone`, or `AckAll`.\n\nAckWait(Int)\n The duration that the server will wait for an ack for any individual\n message once it has been delivered to a consumer. If an ack is not\n received in time, the message will be redelivered.\n\nDeliverPolicy(DeliverPolicy)\n The point in the stream to receive messages from, either `DeliverAll`,\n `DeliverLast`, `DeliverNew`, `DeliverByStartSequence`, `DeliverByStartTime`,\n or `DeliverLastPerSubject`.\n\nInactiveThreshold(Int)\n Duration that instructs the server to cleanup consumers that are inactive\n for that long. Prior to 2.9, this only applied to ephemeral consumers.\n\nMaxAckPending(Int)\n Defines the maximum number of messages, without an acknowledgement, that\n can be outstanding. Once this limit is reached message delivery will be\n suspended. This limit applies across all of the consumer&#39;s bound\n subscriptions. A value of -1 means there can be any number of pending acks\n (i.e. no flow control). This does not apply when the `AckNone` policy is used.\n\nMaxDeliver(Int)\n The maximum number of times a specific message delivery will be attempted.\n Applies to any message that is re-sent due to ack policy (i.e. due to a\n negative ack, or no ack sent by the client).\n\nReplayPolicy(ReplayPolicy)\n If the policy is `ReplayOriginal`, the messages in the stream will be pushed\n to the client at the same rate that they were originally received, simulating\n the original timing of messages. If the policy is `ReplayInstant` (the default),\n the messages will be pushed to the client as fast as possible while adhering\n to the Ack Policy, Max Ack Pending and the client&#39;s ability to consume those\n messages.\n\nNumReplicas(Int)\n Sets the number of replicas for the consumer&#39;s state. By default, when the\n value is set to zero, consumers inherit the number of replicas from the stream.\n\nSampleFrequency(String)\n Sets the percentage of acknowledgements that should be sampled for observability,\n 0-100 This value is a string and for example allows both `30` and `30%` as valid\n values.\n\nMaxWaiting(Int)\n The maximum number of waiting pull requests.\n\nMaxRequestExpires(Int)\n The maximum duration a single pull request will wait for messages to be\n available to pull.\n\nMaxRequestBatch(Int)\n The maximum batch size a single pull request can make. When set with\n `MaxRequestMaxBytes`, the batch size will be constrained by whichever\n limit is hit first.\n\nMaxRequestMaxBytes(Int)\n The maximum total bytes that can be requested in a given batch. When\n set with `MaxRequestBatch`, the batch size will be constrained by whichever\n limit is hit first.\n\nDeliverSubject(String)\n The subject to deliver messages to. Note, setting this field implicitly\n decides whether the consumer is push or pull-based. With a deliver subject,\n the server will push messages to client subscribed to this subject.\n\nDeliverGroup(String)\n The queue group name which, if specified, is then used to distribute the\n messages between the subscribers to the consumer. This is analogous to a\n queue group in core NATS.\n\nHeadersOnly\n Delivers only the headers of messages in the stream and not the bodies.\n Additionally adds Nats-Msg-Size header to indicate the size of the removed\n payload.\n\nSynonyms:\nglats/jetstream/consumer.ConsumerOption\nglats/jetstream/consumer ConsumerOption","url":"glats/jetstream/consumer.html#ConsumerOption"},{"doc":"glats/jetstream/consumer","title":"DeliverPolicy","content":"pub type DeliverPolicy {\n  DeliverAll\n  DeliverLast\n  DeliverLastPerSubject\n  DeliverNew\n  DeliverByStartSequence(Int)\n  DeliverByStartTime(String)\n}\n Available delivier policies to select the point in the stream to\n start consuming from.\n\nDeliverAll\n The default policy. The consumer will start receiving from the earliest\n available message.\n\nDeliverLast\n When first consuming messages, the consumer will start receiving messages\n with the last message added to the stream, or the last message in the\n stream that matches the consumer&#39;s filter subject if defined.\n\nDeliverLastPerSubject\n When first consuming messages, start with the latest one for each filtered\n subject currently in the stream.\n\nDeliverNew\n When first consuming messages, the consumer will only start receiving\n messages that were created after the consumer was created.\n\nDeliverByStartSequence(Int)\n When first consuming messages, start at the first message having the\n sequence number or the next one available.\n\nDeliverByStartTime(String)\n When first consuming messages, start with messages on or after this time.\n\nSynonyms:\nglats/jetstream/consumer.DeliverPolicy\nglats/jetstream/consumer DeliverPolicy","url":"glats/jetstream/consumer.html#DeliverPolicy"},{"doc":"glats/jetstream/consumer","title":"ReplayPolicy","content":"pub type ReplayPolicy {\n  ReplayInstant\n  ReplayOriginal\n}\n The policy to control how to replay messages from a stream.\n\nReplayInstant\n The default policy. The messages will be pushed to the client as fast as\n possible while adhering to the Ack Policy, Max Ack Pending and the client&#39;s\n ability to consume those messages.\n\nReplayOriginal\n The messages in the stream will be pushed to the client at the same rate\n that they were originally received, simulating the original timing of messages.\n\nSynonyms:\nglats/jetstream/consumer.ReplayPolicy\nglats/jetstream/consumer ReplayPolicy","url":"glats/jetstream/consumer.html#ReplayPolicy"},{"doc":"glats/jetstream/consumer","title":"RequestBatchOption","content":"pub type RequestBatchOption {\n  Batch(Int)\n  NoWait\n  Expires(Int)\n}\n Options for `request_batch`.\nBatch(Int)\n The number of messages to receive. Defaults to 1.\n\nNoWait\n Get an empty message immediately if no new ones exist for\n the consumer.\n\nExpires(Int)\n Expiry time for the request in nanoseconds.\n\nSynonyms:\nglats/jetstream/consumer.RequestBatchOption\nglats/jetstream/consumer RequestBatchOption","url":"glats/jetstream/consumer.html#RequestBatchOption"},{"doc":"glats/jetstream/consumer","title":"SequenceInfo","content":"pub type SequenceInfo {\n  SequenceInfo(consumer_seq: Int, stream_seq: Int)\n}\n\nSequenceInfo(consumer_seq: Int, stream_seq: Int)\n\n\nSynonyms:\nglats/jetstream/consumer.SequenceInfo\nglats/jetstream/consumer SequenceInfo","url":"glats/jetstream/consumer.html#SequenceInfo"},{"doc":"glats/jetstream/consumer","title":"Subscription","content":"pub opaque type Subscription\n An active subscription to a consumer.\n\n\nSynonyms:\nglats/jetstream/consumer.Subscription\nglats/jetstream/consumer Subscription","url":"glats/jetstream/consumer.html#Subscription"},{"doc":"glats/jetstream/consumer","title":"SubscriptionOption","content":"pub type SubscriptionOption {\n  Bind(String, String)\n  BindStream(String)\n  With(ConsumerOption)\n}\n Available options to set during subscribe to a stream subject.\n\nBind(String, String)\n Used to bind to an existing stream and consumer while subscribing.\n\nBindStream(String)\n Used to bind to an existing stream.\n\nWith(ConsumerOption)\n When not binding to an existing consumer this can be used to add\n consumer options for the consumer that will be created automatically.\n\nSynonyms:\nglats/jetstream/consumer.SubscriptionOption\nglats/jetstream/consumer SubscriptionOption","url":"glats/jetstream/consumer.html#SubscriptionOption"},{"doc":"glats/jetstream/consumer","title":"create","content":"pub fn create(\n  conn: Subject(ConnectionMessage),\n  stream: String,\n  opts: List(ConsumerOption),\n) -&gt; Result(ConsumerInfo, JetstreamError)\n Creates a new consumer\n\nSynonyms:\nglats/jetstream/consumer.create\nglats/jetstream/consumer create","url":"glats/jetstream/consumer.html#create"},{"doc":"glats/jetstream/consumer","title":"delete","content":"pub fn delete(\n  conn: Subject(ConnectionMessage),\n  stream: String,\n  name: String,\n) -&gt; Result(Nil, JetstreamError)\n Deletes a consumer\n\nSynonyms:\nglats/jetstream/consumer.delete\nglats/jetstream/consumer delete","url":"glats/jetstream/consumer.html#delete"},{"doc":"glats/jetstream/consumer","title":"info","content":"pub fn info(\n  conn: Subject(ConnectionMessage),\n  stream: String,\n  name: String,\n) -&gt; Result(ConsumerInfo, JetstreamError)\n Get info about a consumer by stream and name.\n\nSynonyms:\nglats/jetstream/consumer.info\nglats/jetstream/consumer info","url":"glats/jetstream/consumer.html#info"},{"doc":"glats/jetstream/consumer","title":"names","content":"pub fn names(\n  conn: Subject(ConnectionMessage),\n  stream: String,\n) -&gt; Result(Nil, JetstreamError)\n Get list of consumer names in a stream.\n\nSynonyms:\nglats/jetstream/consumer.names\nglats/jetstream/consumer names","url":"glats/jetstream/consumer.html#names"},{"doc":"glats/jetstream/consumer","title":"request_batch","content":"pub fn request_batch(\n  sub: Subscription,\n  opts: List(RequestBatchOption),\n) -&gt; Result(Nil, JetstreamError)\n Request a batch of messages for a pull subscription.\n\nSynonyms:\nglats/jetstream/consumer.request_batch\nglats/jetstream/consumer request_batch","url":"glats/jetstream/consumer.html#request_batch"},{"doc":"glats/jetstream/consumer","title":"subscribe","content":"pub fn subscribe(\n  conn: Subject(ConnectionMessage),\n  subscriber: Subject(SubscriptionMessage),\n  topic: String,\n  opts: List(SubscriptionOption),\n) -&gt; Result(Subscription, JetstreamError)\n Subscribe to a topic in a stream.\n\n - If no option is provided it will attempt to look up a stream\n   by the topic and create an ephemeral consumer for the\n   subscription.\n - If `Bind(&quot;stream&quot;, &quot;consumer&quot;)` is provided it will subsribe\n   to the stream and existing consumer, failing if either do not\n   exist.\n - If `BindStream(&quot;stream&quot;)` is provided it will not attempt to\n   lookup the stream by topic but creates an ephemeral consumer\n   for the subscription.\n\n In the cases where an ephemeral consumer will be created\n `With(ConsumerOption)` can be provided to configure it.\n\nSynonyms:\nglats/jetstream/consumer.subscribe\nglats/jetstream/consumer subscribe","url":"glats/jetstream/consumer.html#subscribe"},{"doc":"glats/jetstream/consumer","title":"glats/jetstream/consumer","content":"","url":"glats/jetstream/consumer.html"},{"doc":"glats/jetstream/handler","title":"Outcome","content":"pub type Outcome(a) {\n  Ack(a)\n  Nack(a)\n  Term(a)\n  NoReply(a)\n}\n Used to instruct the consumer handler what to do\n with the processed message.\n Also used to return new state.\n\nAck(a)\n Acknowledge message and save state.\n\nNack(a)\n Negatively acknowledge message and save state.\n\nTerm(a)\n Terminate message and save state.\n\nNoReply(a)\n Do nothing with message and save state.\n\nSynonyms:\nglats/jetstream/handler.Outcome\nglats/jetstream/handler Outcome","url":"glats/jetstream/handler.html#Outcome"},{"doc":"glats/jetstream/handler","title":"SubscriptionHandler","content":"pub type SubscriptionHandler(a) =\n  fn(glats.Message, a) -&gt; Outcome(a)\n The handler func that should be passed to the consumer handler.\n\n\nSynonyms:\nglats/jetstream/handler.SubscriptionHandler\nglats/jetstream/handler SubscriptionHandler","url":"glats/jetstream/handler.html#SubscriptionHandler"},{"doc":"glats/jetstream/handler","title":"handle_pull_consumer","content":"pub fn handle_pull_consumer(\n  conn: Subject(ConnectionMessage),\n  initial_state: a,\n  topic: String,\n  batch_size: Int,\n  handler: fn(Message, a) -&gt; Outcome(a),\n  opts: List(SubscriptionOption),\n) -&gt; Result(Subject(SubscriptionMessage), StartError)\n Start a pull consumer handler actor.\n\nSynonyms:\nglats/jetstream/handler.handle_pull_consumer\nglats/jetstream/handler handle_pull_consumer","url":"glats/jetstream/handler.html#handle_pull_consumer"},{"doc":"glats/jetstream/handler","title":"glats/jetstream/handler","content":" A convenience handler that will handle a consumer subscription for you.\n For every message it receives it will call the provided `SubscriptionHandler(a)`\n function and take action depending on its return value.\n\n It will also keep the state for you of type `a`.\n\n ## Pull consumer example\n\n ```gleam\n import gleam/io\n import gleam/int\n import gleam/string\n import gleam/result\n import gleam/function\n import gleam/erlang/process\n import glats.{Connection, Message}\n import glats/jetstream/stream.{Retention, WorkQueuePolicy}\n import glats/jetstream/consumer.{\n   AckExplicit, AckPolicy, BindStream, Description, With,\n }\n import glats/jetstream/handler.{Ack}\n \n pub fn main() {\n   use conn &lt;- result.then(glats.connect(&quot;localhost&quot;, 4222, []))\n \n   // Create a stream\n   let assert Ok(stream) =\n     stream.create(conn, &quot;wqstream&quot;, [&quot;ticket.&gt;&quot;], [Retention(WorkQueuePolicy)])\n \n   // Run pull handler\n   let assert Ok(_actor) =\n     handler.handle_pull_consumer(\n       conn,\n       0,            // Initial state\n       &quot;ticket.*&quot;,   // Topic\n       100,          // Batch size\n       pull_handler, // Handler function\n       [\n         // Bind to stream created above\n         BindStream(stream.config.name),\n         // Set description for the ephemeral consumer\n         With(Description(&quot;An ephemeral consumer for subscription&quot;)),\n         // Set ack policy for the consumer\n         With(AckPolicy(AckExplicit)),\n       ],\n     )\n \n   // Run a loop that publishes a message every 100ms\n   publish_loop(conn, 0)\n \n   Ok(Nil)\n }\n \n // Publishes a new message every 100ms\n fn publish_loop(conn: Connection, counter: Int) {\n   let assert Ok(Nil) =\n     glats.publish(\n       conn,\n       &quot;ticket.&quot; &lt;&gt; int.to_string(counter),\n       &quot;ticket body&quot;,\n       [],\n     )\n \n   process.sleep(100)\n \n   publish_loop(conn, counter + 1)\n }\n \n // Handler function for the pull consumer handler\n pub fn pull_handler(message: Message, state) {\n   // Increment state counter, print message and instruct\n   // pull handler to ack the message.\n   state + 1\n   |&gt; function.tap(print_message(_, message.topic, message.body))\n   |&gt; Ack\n }\n\n fn print_message(num: Int, topic: String, body: String) {\n   &quot;message &quot; &lt;&gt; int.to_string(num) &lt;&gt; &quot; (&quot; &lt;&gt; topic &lt;&gt; &quot;): &quot; &lt;&gt; body\n   |&gt; io.println\n }\n ```\n\n Will output:\n\n ```sh\n message 1 (ticket.0): ticket body\n message 2 (ticket.1): ticket body\n message 3 (ticket.2): ticket body\n message 4 (ticket.3): ticket body\n message 5 (ticket.4): ticket body\n ...\n ```","url":"glats/jetstream/handler.html"}]);